@@ -875,12 +875,30 @@ class Channel (ClosingContextManager):
         client, it only makes sense to open this file for reading.  For a
         server, it only makes sense to open this file for writing.
 
-        :return: `.ChannelFile` object which can be used for Python file I/O.
+        :returns:
+            `.ChannelStderrFile` object which can be used for Python file I/O.
 
         .. versionadded:: 1.1
         """
         return ChannelStderrFile(*([self] + list(params)))
 
+    def makefile_stdin(self, *params):
+        """
+        Return a file-like object associated with this channel's stdin
+        stream.
+
+        The optional ``mode`` and ``bufsize`` arguments are interpreted the
+        same way as by the built-in ``file()`` function in Python.  For a
+        client, it only makes sense to open this file for writing.  For a
+        server, it only makes sense to open this file for reading.
+
+        :returns:
+            `.ChannelStdinFile` object which can be used for Python file I/O.
+
+        .. versionadded:: 2.6
+        """
+        return ChannelStdinFile(*([self] + list(params)))
+
     def fileno(self):
         """
         Returns an OS-level file descriptor which can be used for polling, but
@@ -926,6 +944,7 @@ class Channel (ClosingContextManager):
             # feign "read" shutdown
             self.eof_received = 1
         if (how == 1) or (how == 2):
+            m = None
             self.lock.acquire()
             try:
                 m = self._send_eof()
@@ -990,7 +1009,7 @@ class Channel (ClosingContextManager):
             'Max packet out: {} bytes'.format(self.out_max_packet_size))
 
     def _request_success(self, m):
-        self._log(DEBUG, 'Sesch channel {} request ok'.format(self.chanid))
+        self._log(DEBUG, 'Secsh channel {} request ok'.format(self.chanid))
         self.event_ready = True
         self.event.set()
         return
@@ -1034,7 +1053,7 @@ class Channel (ClosingContextManager):
             if self.ultra_debug:
                 self._log(DEBUG, 'window up {}'.format(nbytes))
             self.out_window_size += nbytes
-            self.out_buffer_cv.notify_all()
+            self.out_buffer_cv.notifyAll()
         finally:
             self.lock.release()
 
@@ -1203,7 +1222,7 @@ class Channel (ClosingContextManager):
         self.closed = True
         self.in_buffer.close()
         self.in_stderr_buffer.close()
-        self.out_buffer_cv.notify_all()
+        self.out_buffer_cv.notifyAll()
         # Notify any waiters that we are closed
         self.event.set()
         self.status_event.set()
@@ -1212,7 +1231,7 @@ class Channel (ClosingContextManager):
 
     def _send_eof(self):
         # you are holding the lock.
-        if self.eof_sent:
+        if self.eof_sent or self.closed:
             return None
         m = Message()
         m.add_byte(cMSG_CHANNEL_EOF)
@@ -1337,9 +1356,12 @@ class ChannelFile (BufferedFile):
         return len(data)
 
 
-class ChannelStderrFile (ChannelFile):
-    def __init__(self, channel, mode='r', bufsize=-1):
-        ChannelFile.__init__(self, channel, mode, bufsize)
+class ChannelStderrFile(ChannelFile):
+    """
+    A file-like wrapper around `.Channel` stderr.
+
+    See `Channel.makefile_stderr` for details.
+    """
 
     def _read(self, size):
         return self.channel.recv_stderr(size)
@@ -1349,4 +1371,13 @@ class ChannelStderrFile (ChannelFile):
         return len(data)
 
 
-# vim: set shiftwidth=4 expandtab :
+class ChannelStdinFile(ChannelFile):
+    """
+    A file-like wrapper around `.Channel` stdin.
+
+    See `Channel.makefile_stdin` for details.
+    """
+
+    def close(self):
+        super(ChannelStdinFile, self).close()
+        self.channel.shutdown_write()
