@@ -31,11 +31,12 @@ from paramiko.agent import Agent
 from paramiko.common import DEBUG
 from paramiko.config import SSH_PORT
 from paramiko.dsskey import DSSKey
+from paramiko.rsakey import RSAKey
 from paramiko.ecdsakey import ECDSAKey
 from paramiko.ed25519key import Ed25519Key
+from paramiko.pkey import load_private_key_file
 from paramiko.hostkeys import HostKeys
 from paramiko.py3compat import string_types
-from paramiko.rsakey import RSAKey
 from paramiko.ssh_exception import SSHException, BadHostKeyException, BadAuthenticationType
 from paramiko.transport import Transport
 from paramiko.util import retry_on_signal, ClosingContextManager
@@ -436,7 +437,7 @@ class SSHClient (ClosingContextManager):
         if environment:
             chan.update_environment(environment)
         chan.exec_command(command)
-        stdin = chan.makefile('wb', bufsize)
+        stdin = chan.makefile_stdin('wb', bufsize)
         stdout = chan.makefile('r', bufsize)
         stderr = chan.makefile_stderr('r', bufsize)
         return stdin, stdout, stderr
@@ -484,7 +485,7 @@ class SSHClient (ClosingContextManager):
         """
         return self._transport
 
-    def _key_from_filepath(self, filename, klass, password):
+    def _key_from_filepath(self, filename, klass=None, password=None):
         """
         Attempt to derive a `.PKey` from given string path ``filename``:
 
@@ -502,12 +503,15 @@ class SSHClient (ClosingContextManager):
             key_path = filename
             cert_path = filename + cert_suffix
         # Blindly try the key path; if no private key, nothing will work.
-        key = klass.from_private_key_file(key_path, password)
+        if klass:
+            key = klass.from_private_key_file(key_path, password)
+        else:
+            key = load_private_key_file(key_path, password)
         # TODO: change this to 'Loading' instead of 'Trying' sometime; probably
         # when #387 is released, since this is a critical log message users are
         # likely testing/filtering for (bah.)
         msg = "Trying discovered key {} in {}".format(
-            hexlify(key.get_fingerprint()), key_path,
+            hexlify(key.get_fingerprint()).decode(), key_path,
         )
         self._log(DEBUG, msg)
         # Attempt to load cert if it exists.
@@ -577,10 +581,7 @@ class SSHClient (ClosingContextManager):
 
         if pkey is not None and 'publickey' in allowed_types:
             try:
-                self._log(
-                    DEBUG,
-                    'Trying SSH key {}'.format(hexlify(pkey.get_fingerprint()))
-                )
+                self._log(DEBUG, "Trying SSH key %s" % hexlify(pkey.get_fingerprint()).decode())
                 allowed_types = set(
                     self._transport.auth_publickey(username, pkey))
                 two_factor = (allowed_types & two_factor_types)
@@ -591,19 +592,16 @@ class SSHClient (ClosingContextManager):
 
         if not two_factor and 'publickey' in allowed_types:
             for key_filename in key_filenames:
-                for pkey_class in (RSAKey, DSSKey, ECDSAKey, Ed25519Key):
-                    try:
-                        key = self._key_from_filepath(
-                            key_filename, pkey_class, passphrase,
-                        )
-                        allowed_types = set(
-                            self._transport.auth_publickey(username, key))
-                        two_factor = (allowed_types & two_factor_types)
-                        if not two_factor:
-                            return
-                        break
-                    except SSHException as e:
-                        saved_exception = e
+                try:
+                    key = self._key_from_filepath(key_filename, password=passphrase)
+
+                    allowed_types = set(self._transport.auth_publickey(username, key))
+                    two_factor = (allowed_types & two_factor_types)
+                    if not two_factor:
+                        return
+                    break
+                except SSHException as e:
+                    saved_exception = e
 
         if allow_agent and not two_factor and 'publickey' in allowed_types:
             if self._agent is None:
@@ -611,8 +609,8 @@ class SSHClient (ClosingContextManager):
 
             for key in self._agent.get_keys():
                 try:
-                    id_ = hexlify(key.get_fingerprint())
-                    self._log(DEBUG, 'Trying SSH agent key {}'.format(id_))
+                    self._log(DEBUG, "Trying SSH agent key %s"
+                              % hexlify(key.get_fingerprint()).decode())
                     allowed_types = set(
                         self._transport.auth_publickey(username, key))
                     two_factor = (allowed_types & two_factor_types)
@@ -683,7 +681,7 @@ class SSHClient (ClosingContextManager):
         raise SSHException('No authentication methods available')
 
     def _log(self, level, msg):
-        self._transport._log(level, msg)
+        self._transport._log(level, "%s", msg)
 
 
 class MissingHostKeyPolicy (object):
@@ -718,7 +716,7 @@ class AutoAddPolicy (MissingHostKeyPolicy):
         if client._host_keys_filename is not None:
             client.save_host_keys(client._host_keys_filename)
         client._log(DEBUG, 'Adding {} host key for {}: {}'.format(
-            key.get_name(), hostname, hexlify(key.get_fingerprint()),
+            key.get_name(), hostname, hexlify(key.get_fingerprint()).decode(),
         ))
 
 
@@ -730,7 +728,7 @@ class RejectPolicy (MissingHostKeyPolicy):
 
     def missing_host_key(self, client, hostname, key):
         client._log(DEBUG, 'Rejecting {} host key for {}: {}'.format(
-            key.get_name(), hostname, hexlify(key.get_fingerprint()),
+            key.get_name(), hostname, hexlify(key.get_fingerprint()).decode(),
         ))
         raise SSHException(
             'Server {!r} not found in known_hosts'.format(hostname)
@@ -744,5 +742,5 @@ class WarningPolicy (MissingHostKeyPolicy):
     """
     def missing_host_key(self, client, hostname, key):
         warnings.warn('Unknown {} host key for {}: {}'.format(
-            key.get_name(), hostname, hexlify(key.get_fingerprint()),
+            key.get_name(), hostname, hexlify(key.get_fingerprint()).decode(),
         ))
