@@ -17,13 +17,13 @@
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 
 """
-Common API for all public keys.
+Common API for all public and private keys.
 """
 
 import base64
 from binascii import unhexlify
 import os
-from hashlib import md5, sha256
+from hashlib import md5
 import re
 
 import bcrypt
@@ -34,12 +34,95 @@ from cryptography.hazmat.primitives.ciphers import algorithms, modes, Cipher
 
 from paramiko import util
 from paramiko.common import o600
-from paramiko.py3compat import u, encodebytes, decodebytes, b, string_types, byte_ord, PY2
+from paramiko.py3compat import u, encodebytes, decodebytes, b, string_types, byte_ord
 from paramiko.ssh_exception import SSHException, PasswordRequiredException
 from paramiko.message import Message
 
 
-def unpad(data):
+_legacy_type_registry = {}
+_openssh_type_registry = []
+
+
+def load_private_key(key_str, password=None):
+    """
+    Create a key object by reading a private key from a string.  If the private
+    key is encrypted and ``password`` is not ``None``, the given password will
+    be used to decrypt the key (otherwise `.PasswordRequiredException` is
+    thrown).
+
+    The key type is auto-detected and an instance of the appropriate `PKey` type
+    is returned. This is now the recommended API to load private keys, instead
+    of instantiating a ``*Key`` class directly.
+
+    :param str key_str: string containing the key
+    :param str password:
+        an optional password to use to decrypt the key, if it's encrypted
+    :return: `.PKey` based on the type detected
+
+    :raises: `.PasswordRequiredException` --
+        if the private key is encrypted, and ``password`` is ``None``
+    :raises: `.SSHException` -- if the key file is invalid
+
+    .. versionadded:: 2.8
+    """
+    pkformat, typ, data = PKey._read_private_key(key_str, password)
+
+    cls = None
+    if pkformat == PKey.FORMAT_ORIGINAL:
+        cls = _legacy_type_registry.get(typ)
+    elif pkformat == PKey.FORMAT_OPENSSH:
+        for map_type, map_cls in _openssh_type_registry:
+            if typ.startswith(map_type):
+                cls = map_cls
+    else:
+        raise AssertionError("Unexpected pkformat {}".format(pkformat))  # impossible
+
+    if cls is None:
+        raise SSHException("Unsupported key type {}".format(typ))
+
+    return cls(_raw=(pkformat, data))
+
+
+def load_private_key_file(filename, password=None):
+    """
+    Create a key object by reading a private key from file.  If the private key
+    is encrypted and ``password`` is not ``None``, the given password will be
+    used to decrypt the key (otherwise `.PasswordRequiredException` is thrown).
+
+    The key type is auto-detected and an instance of the appropriate `PKey`
+    subclass is returned. This is now the recommended API to load private keys,
+    instead of instantiating a ``*Key`` class directly.
+
+    :param str filename: path to the file to load
+    :param str password:
+        an optional password to use to decrypt the key, if it's encrypted
+    :return: `.PKey` based on the given private key
+
+    :raises: ``IOError`` -- if there was an error opening or reading the file
+    :raises: `.PasswordRequiredException` --
+        if the private key file is encrypted, and ``password`` is ``None``
+    :raises: `.SSHException` -- if the key file is invalid
+
+    .. versionadded:: 2.8
+    """
+    with open(filename, "r") as f:
+        return load_private_key(f.read(), password)
+
+
+def register_pkey_type(cls):
+    """
+    Decorator for PKey subclasses to register their types for parsing.
+
+    .. versionadded:: 2.8
+    """
+    if cls.LEGACY_TYPE:
+        _legacy_type_registry[cls.LEGACY_TYPE] = cls
+    if cls.OPENSSH_TYPE_PREFIX:
+        _openssh_type_registry.append((cls.OPENSSH_TYPE_PREFIX, cls))
+    return cls
+
+
+def _unpad(data):
     """for removing padding from openssh new-format private key files"""
     padding_length = byte_ord(data[-1])
     if 0x20 <= padding_length < 0x7f:
@@ -78,10 +161,24 @@ class PKey(object):
             'mode': modes.CBC
         },
     }
+
     FORMAT_ORIGINAL = 1  # PKCS#1 for RSA or SEC1 for EC
     FORMAT_OPENSSH = 2  # newer "openssh-key-v1" format
-    BEGIN_TAG = re.compile(r'^-{5}BEGIN (RSA|DSA|EC|OPENSSH) PRIVATE KEY-{5}\s*$')
-    END_TAG = re.compile(r'^-{5}END (RSA|DSA|EC|OPENSSH) PRIVATE KEY-{5}\s*$')
+
+    _OPENSSH_KEY_RE = re.compile(
+        r"^-{5}BEGIN (.+) PRIVATE KEY-{5}\s*\n"
+        r"((?:.+: .+\r?\n)*)\s*"  # headers (original only, optional)
+        r"([0-9A-Za-z+/=\s]+)"  # body (multi-line base64)
+        r"-{5}END \1 PRIVATE KEY-{5}\s*$",
+        re.MULTILINE
+    )
+
+    #: Subclasses set this to identify the key type in `.FORMAT_ORIGINAL` files.
+    #: Examples: ``"RSA"``, ``"EC"``
+    LEGACY_TYPE = None
+    #: Subclasses set this to identify the key type in `.FORMAT_OPENSSH` files.
+    #: Examples: ``"ssh-rsa"``, ``"ecdsa-sha2-"``
+    OPENSSH_TYPE_PREFIX = None
 
     def __init__(self, msg=None, data=None):
         """
@@ -103,36 +200,35 @@ class PKey(object):
 
     def asbytes(self):
         """
-        Return bytes of an SSH `.Message` made up of the public part(s) of
+        Return a string of an SSH `.Message` made up of the public part(s) of
         this key.  This string is suitable for passing to `__init__` to
         re-create the key object later.
         """
-        raise NotImplementedError()
+        return bytes()
 
     def __str__(self):
-        if PY2:
-            # strange original behavior for backwards compatibility
-            return self.asbytes()
-        else:
-            fingerprint = sha256(self.asbytes()).digest()
-            fingerprint_b64 = u(base64.b64encode(fingerprint)[:-1])
-            return "%s SHA256:%s" % (self.get_name(), fingerprint_b64)
+        return self.asbytes()
 
+    # noinspection PyUnresolvedReferences
+    # TODO: The comparison functions should be removed as per:
+    # https://docs.python.org/3.0/whatsnew/3.0.html#ordering-comparisons
     def __cmp__(self, other):
-        # python-2 only, same purpose as __eq__()
-        other_asbytes = other.asbytes() if isinstance(other, PKey) else b""
-        return cmp(self.asbytes(), other_asbytes)  # noqa
-
-    def __eq__(self, other):
         """
-        Compare this key to another.  Returns True if this key is equivalent to
-        the given key, or False if they are different.  Only the public parts
+        Compare this key to another.  Returns 0 if this key is equivalent to
+        the given key, or non-0 if they are different.  Only the public parts
         of the key are compared, so a public key will compare equal to its
         corresponding private key.
 
         :param .PKey other: key to compare to.
         """
-        return isinstance(other, PKey) and self.asbytes() == other.asbytes()
+        hs = hash(self)
+        ho = hash(other)
+        if hs != ho:
+            return cmp(hs, ho)   # noqa
+        return cmp(self.asbytes(), other.asbytes())  # noqa
+
+    def __eq__(self, other):
+        return hash(self) == hash(other)
 
     def get_name(self):
         """
@@ -142,7 +238,7 @@ class PKey(object):
             name of this private key type, in SSH terminology, as a `str` (for
             example, ``"ssh-rsa"``).
         """
-        raise NotImplementedError()
+        return ''
 
     def get_bits(self):
         """
@@ -213,6 +309,10 @@ class PKey(object):
         exist in all subclasses of PKey (such as `.RSAKey` or `.DSSKey`), but
         is useless on the abstract PKey class.
 
+        .. note::
+            It is recommended to use the key-type-agnostic
+            `.load_private_key_file` function instead.
+
         :param str filename: name of the file to read
         :param str password:
             an optional password to use to decrypt the key file, if it's
@@ -235,6 +335,10 @@ class PKey(object):
         ``None``, the given password will be used to decrypt the key (otherwise
         `.PasswordRequiredException` is thrown).
 
+        .. note::
+            It is recommended to use the key-type-agnostic `.load_private_key`
+            function instead.
+
         :param file_obj: the file-like object to read from
         :param str password:
             an optional password to use to decrypt the key, if it's encrypted
@@ -275,79 +379,81 @@ class PKey(object):
         """
         raise Exception('Not implemented in PKey')
 
-    def _read_private_key_file(self, tag, typepfx, filename, password=None):
-        """
-        Read an SSH2-format private key file, looking for a string of the type
+    @classmethod
+    def _parse_openssh_pkey(cls, key_str):
+        """Common parsing logic for PEM-like OpenSSH private key formats.
+
+        Parse SSH2-format private string, looking for line containing the marker
         ``"BEGIN xxx PRIVATE KEY"`` for some ``xxx``, base64-decode the text we
-        find, and return it as a string.  If the private key is encrypted and
-        ``password`` is not ``None``, the given password will be used to
-        decrypt the key (otherwise `.PasswordRequiredException` is thrown).
+        find, and return it along with some metadata. For ``FORMAT_ORIGINAL``
+        keys, encryption headers are also parsed and returned as dict.
 
-        :param str tag: ``"RSA"`` or ``"DSA"``, the tag used to mark the data block
-            (in FORMAT_ORIGINAL files)
-        :param str typepfx: key type name prefix, e.g. ``"ssh-rsa"`` or ``"ecdsa-sha2-"``
-            (in FORMAT_OPENSSH files)
-        :param str filename: name of the file to read.
-        :param str password: optional, for decrypting the key file, if it's encrypted
-        :return: data blob (`str`) that makes up the private key.
+        :return: Four-tuple with the following fields:
 
-        :raises: ``IOError`` -- if there was an error reading the file.
-        :raises: `.PasswordRequiredException` -- if the private key file is
-            encrypted, and ``password`` is ``None``.
-        :raises: `.SSHException` -- if the key file is invalid.
+            * pkformat: ``FORMAT_ORIGINAL`` or ``FORMAT_OPENSSH``.
+            * type: string containing the type from BEGIN tag (e.g. ``RSA``).
+            * headers: encryption headers if it's a FORMAT_ORIGINAL key,
+              otherwise an empty ``dict``.
+            * data: base64-decoded data from the key body.
         """
-        with open(filename, 'r') as f:
-            data = self._read_private_key(tag, typepfx, f, password)
-        return data
+        match = cls._OPENSSH_KEY_RE.search(key_str)
+        if not match:
+            raise SSHException("not a valid private key file")
 
-    def _read_private_key(self, tag, typepfx, f, password=None):
-        lines = f.readlines()
+        typ, header_str, body = match.groups()
 
-        # find the BEGIN tag
-        start = 0
-        m = self.BEGIN_TAG.match(lines[start])
-        line_range = len(lines) - 1
-        while start < line_range and not m:
-            start += 1
-            m = self.BEGIN_TAG.match(lines[start])
-        start += 1
-        keytype = m.group(1) if m else None
-        if start >= len(lines) or keytype is None:
-            raise SSHException('not a valid ' + tag + ' private key file')
-
-        # find the END tag
-        end = start
-        m = self.END_TAG.match(lines[end])
-        while end < line_range and not m:
-            end += 1
-            m = self.END_TAG.match(lines[end])
-
-        if keytype == 'OPENSSH':
-            data = self._read_private_key_new_format(typepfx, lines[start:end], password)
-            pkformat = self.FORMAT_OPENSSH
-        elif keytype == tag:
-            data = self._read_private_key_old_format(lines[start:end], password)
-            pkformat = self.FORMAT_ORIGINAL
+        headers = {}
+        if typ == "OPENSSH":
+            if header_str:
+                raise SSHException("OPENSSH-format key should not contain headers")
+            pkformat = cls.FORMAT_OPENSSH
         else:
-            raise SSHException('encountered {} key, expected {} key'.format(keytype, tag))
+            pkformat = cls.FORMAT_ORIGINAL
+            if header_str:
+                for line in header_str.splitlines():
+                    key, value = line.split(": ", 1)
+                    headers[key.lower()] = value.strip()
+
+        try:
+            data = decodebytes(b(body))
+        except base64.binascii.Error as e:
+            raise SSHException("base64 decoding error: " + str(e))
+
+        return pkformat, typ, headers, data
+
+    def _from_private_key_file(self, filename, password=None):
+        with open(filename, 'r') as file_obj:
+            return self._from_private_key(file_obj, password)
+
+    def _from_private_key(self, file_obj, password=None):
+        pkformat, typ, data = self._read_private_key(file_obj.read(), password)
+
+        if pkformat == self.FORMAT_ORIGINAL:
+            if typ != self.LEGACY_TYPE:
+                raise SSHException("Expected key type {}, got {}"
+                                   .format(self.LEGACY_TYPE or "OPENSSH", typ))
+
+        elif pkformat == self.FORMAT_OPENSSH:
+            if not typ.startswith(self.OPENSSH_TYPE_PREFIX):
+                raise SSHException("Expected key type {}, got {}"
+                                   .format(self.OPENSSH_TYPE_PREFIX, typ))
+        else:
+            raise AssertionError("Unexpected pkformat {}".format(pkformat))  # impossible
 
         return pkformat, data
 
-    def _read_private_key_old_format(self, lines, password):
-        # parse any headers first
-        start = 0
-        headers = {}
-        while start < len(lines):
-            l = lines[start].split(': ')
-            if len(l) == 1:
-                break
-            headers[l[0].lower()] = l[1].strip()
-            start += 1
-        # if we trudged to the end of the file, just try to cope.
-        try:
-            data = decodebytes(b(''.join(lines[start:])))
-        except base64.binascii.Error as e:
-            raise SSHException('base64 decoding error: ' + str(e))
+    @classmethod
+    def _read_private_key(cls, key_str, password):
+        pkformat, typ, headers, data = cls._parse_openssh_pkey(key_str)
+        if pkformat == cls.FORMAT_ORIGINAL:
+            data = cls._read_private_key_old_format(headers, data, password)
+        elif pkformat == cls.FORMAT_OPENSSH:
+            assert not headers
+            typ, data = cls._read_private_key_new_format(data, password)
+        return pkformat, typ, data
+
+    @classmethod
+    def _read_private_key_old_format(cls, headers, data, password):
         if 'proc-type' not in headers:
             # unencryped: done
             return data
@@ -361,16 +467,16 @@ class PKey(object):
             encryption_type, saltstr = headers['dek-info'].split(',')
         except:
             raise SSHException("Can't parse DEK-info in private key file")
-        if encryption_type not in self._CIPHER_TABLE:
+        if encryption_type not in cls._CIPHER_TABLE:
             raise SSHException(
                 'Unknown private key cipher "{}"'.format(encryption_type))
         # if no password was passed in,
         # raise an exception pointing out that we need one
         if password is None:
             raise PasswordRequiredException('Private key file is encrypted')
-        cipher = self._CIPHER_TABLE[encryption_type]['cipher']
-        keysize = self._CIPHER_TABLE[encryption_type]['keysize']
-        mode = self._CIPHER_TABLE[encryption_type]['mode']
+        cipher = cls._CIPHER_TABLE[encryption_type]['cipher']
+        keysize = cls._CIPHER_TABLE[encryption_type]['keysize']
+        mode = cls._CIPHER_TABLE[encryption_type]['mode']
         salt = unhexlify(b(saltstr))
         key = util.generate_key_bytes(md5, salt, password, keysize)
         decryptor = Cipher(
@@ -378,7 +484,8 @@ class PKey(object):
         ).decryptor()
         return decryptor.update(data) + decryptor.finalize()
 
-    def _read_private_key_new_format(self, typepfx, lines, password):
+    @staticmethod
+    def _read_private_key_new_format(data, password):
         """
         Read the new OpenSSH SSH2 private key format available
         since OpenSSH version 6.5
@@ -386,11 +493,6 @@ class PKey(object):
         https://github.com/openssh/openssh-portable/blob/master/PROTOCOL.key
         https://coolaj86.com/articles/the-openssh-private-key-format/
         """
-        try:
-            data = decodebytes(b(''.join(lines)))
-        except base64.binascii.Error as e:
-            raise SSHException('base64 decoding error: ' + str(e))
-
         message = Message(data)
         OPENSSH_AUTH_MAGIC = b"openssh-key-v1\x00"
         if message.get_bytes(len(OPENSSH_AUTH_MAGIC)) != OPENSSH_AUTH_MAGIC:
@@ -407,9 +509,7 @@ class PKey(object):
         public_data = message.get_binary()
         privkey_blob = message.get_binary()
 
-        keytype = Message(public_data).get_text()
-        if not keytype.startswith(typepfx):
-            raise SSHException("Invalid key type name (public part)")
+        pub_keytype = Message(public_data).get_text()
 
         if kdfname == "none":
             if kdfoptions or cipher != "none":
@@ -449,11 +549,11 @@ class PKey(object):
             raise SSHException('OpenSSH private key file checkints do not match')
 
         keytype = priv_msg.get_text()
-        if not keytype.startswith(typepfx):
-            raise SSHException("Invalid key type name (private part)")
+        if pub_keytype != keytype:
+            raise SSHException("Inconsistent key types for public and private parts")
 
         keydata = priv_msg.get_remainder()
-        return unpad(keydata)
+        return keytype, _unpad(keydata)
 
     def _write_private_key_file(self, filename, key, format, password=None):
         """
@@ -578,7 +678,7 @@ class PublicBlob(object):
     Tries to be as dumb as possible and barely cares about specific
     per-key-type data.
 
-    ..note::
+    .. note::
         Most of the time you'll want to call `from_file`, `from_string` or
         `from_message` for useful instantiation, the main constructor is
         basically "I should be using ``attrs`` for this."
