@@ -205,6 +205,11 @@ class ClientTest(unittest.TestCase):
         stdin, stdout, stderr = self.tc.exec_command('yes')
         schan = self.ts.accept(1.0)
 
+        # Nobody else tests the API of exec_command so let's do it here for now
+        assert isinstance(stdin, paramiko.ChannelStdinFile)
+        assert isinstance(stdout, paramiko.ChannelFile)
+        assert isinstance(stderr, paramiko.ChannelStderrFile)
+
         schan.send('Hello there.\n')
         schan.send_stderr('This is on stderr.\n')
         schan.close()
@@ -404,17 +409,12 @@ class SSHClientTest(ClientTest):
         self.tc.close()
         del self.tc
 
-        # GC is unpredictable, depending on python version and implementation
-        time.sleep(0.1)
+        # force a collection to see whether the SSHClient object is deallocated
+        # 2 GCs are needed on PyPy, time is needed for Python 3
+        time.sleep(0.3)
         gc.collect()
-        time.sleep(0.2)
-        gc.collect()
-        time.sleep(0.1)
-        gc.collect()
-        time.sleep(0.2)
-        gc.collect()
-        time.sleep(0.1)
         gc.collect()
+
         self.assertTrue(p() is None)
 
     def test_client_can_be_used_as_context_manager(self):
@@ -601,7 +601,7 @@ class SSHClientTest(ClientTest):
         self.tc.connect(self.addr, self.port, username='slowdive', password='pygmalion')
 
         self.event.wait(1.0)
-        self.assertTrue(self.event.is_set())
+        self.assertTrue(self.event.isSet())
         self.assertTrue(self.ts.is_active())
 
     def test_update_environment(self):
