@@ -28,8 +28,21 @@ from hashlib import md5
 
 import pytest
 
-from paramiko import RSAKey, DSSKey, ECDSAKey, Ed25519Key, Message, util
-from paramiko.py3compat import StringIO, byte_chr, b
+from paramiko import (
+    util,
+    RSAKey,
+    DSSKey,
+    ECDSAKey,
+    Ed25519Key,
+    Message,
+    SSHException,
+)
+from paramiko.pkey import (
+    PKey,
+    load_private_key,
+    load_private_key_file,
+)
+from paramiko.py3compat import StringIO, byte_chr, b, PY2
 
 from .util import _support
 
@@ -114,8 +127,23 @@ L4QLcT5aND0EHZLB2fAUDXiWIb2j4rg1mwPlBMiBXA==
 -----END EC PRIVATE KEY-----
 """
 
+UNSUPPORTED_PRIVATE_KEY = """\
+-----BEGIN UNSUPPORTED PRIVATE KEY-----
+BLABLABLABLABLA=
+-----END UNSUPPORTED PRIVATE KEY-----
+"""
+
+INVALID_BASE64_KEY = """\
+-----BEGIN OPENSSH PRIVATE KEY-----
+INVALID/PADDING
+-----END OPENSSH PRIVATE KEY-----
+"""
+
 x1234 = b'\x01\x02\x03\x04'
 
+TEST_KEY_BYTESTR_2 = '\x00\x00\x00\x07ssh-rsa\x00\x00\x00\x01#\x00\x00\x00\x81\x00\xd3\x8fV\xea\x07\x85\xa6k%\x8d<\x1f\xbc\x8dT\x98\xa5\x96$\xf3E#\xbe>\xbc\xd2\x93\x93\x87f\xceD\x18\xdb \x0c\xb3\xa1a\x96\xf8e#\xcc\xacS\x8a#\xefVlE\x83\x1epv\xc1o\x17M\xef\xdf\x89DUXL\xa6\x8b\xaa<\x06\x10\xd7\x93w\xec\xaf\xe2\xaf\x95\xd8\xfb\xd9\xbfw\xcb\x9f0)#y{\x10\x90\xaa\x85l\tPru\x8c\t\x19\xce\xa0\xf1\xd2\xdc\x8e/\x8b\xa8f\x9c0\xdey\x84\xd2F\xf7\xcbmm\x1f\x87'  # noqa: E501
+TEST_KEY_BYTESTR_3 = '\x00\x00\x00\x07ssh-rsa\x00\x00\x00\x01#\x00\x00\x00\x00ӏV\x07k%<\x1fT$E#>ғfD\x18 \x0cae#̬S#VlE\x1epvo\x17M߉DUXL<\x06\x10דw\u2bd5ٿw˟0)#y{\x10l\tPru\t\x19Π\u070e/f0yFmm\x1f'  # noqa: E501
+
 
 class KeyTest(unittest.TestCase):
 
@@ -158,9 +186,6 @@ class KeyTest(unittest.TestCase):
         key2 = RSAKey.from_private_key(s)
         self.assertEqual(key, key2)
 
-        # ensure comparison does not raise exception
-        assert key != None  # noqa: E711
-
     def test_load_rsa_password(self):
         key = RSAKey.from_private_key_file(_support('test_rsa_password.key'), 'television')
         self.assertEqual('ssh-rsa', key.get_name())
@@ -186,9 +211,6 @@ class KeyTest(unittest.TestCase):
         key2 = DSSKey.from_private_key(s)
         self.assertEqual(key, key2)
 
-        # ensure comparison does not raise exception
-        assert key != None  # noqa: E711
-
     def test_load_dss_password(self):
         key = DSSKey.from_private_key_file(_support('test_dss_password.key'), 'television')
         self.assertEqual('ssh-dss', key.get_name())
@@ -301,9 +323,6 @@ class KeyTest(unittest.TestCase):
         key2 = ECDSAKey.from_private_key(s)
         self.assertEqual(key, key2)
 
-        # ensure comparison does not raise exception
-        assert key != None  # noqa: E711
-
     def test_load_ecdsa_password_256(self):
         key = ECDSAKey.from_private_key_file(_support('test_ecdsa_password_256.key'),
                                              b'television')
@@ -498,6 +517,11 @@ class KeyTest(unittest.TestCase):
         finally:
             os.remove(newfile)
 
+    def test_stringification(self):
+        key = RSAKey.from_private_key_file(_support('test_rsa.key'))
+        comparable = TEST_KEY_BYTESTR_2 if PY2 else TEST_KEY_BYTESTR_3
+        self.assertEqual(str(key), comparable)
+
     @pytest.mark.skipif("not Ed25519Key.is_supported()")
     def test_ed25519(self):
         key1 = Ed25519Key.from_private_key_file(_support('test_ed25519.key'))
@@ -506,9 +530,6 @@ class KeyTest(unittest.TestCase):
         )
         self.assertNotEqual(key1.asbytes(), key2.asbytes())
 
-        # ensure comparison does not raise exception
-        assert key1 != None  # noqa: E711
-
     @pytest.mark.skipif("not Ed25519Key.is_supported()")
     def test_ed25519_nopad(self):
         Ed25519Key.from_private_key_file(_support("test_ed25519_nopad.key"))
@@ -590,3 +611,158 @@ class KeyTest(unittest.TestCase):
             key1.load_certificate,
             _support('test_rsa.key-cert.pub'),
         )
+
+    def test_autodetect_ed25519(self):
+        key = load_private_key_file(_support("test_ed25519.key"))
+        self.assertIsInstance(key, Ed25519Key)
+
+    def test_autodetect_ecdsa(self):
+        key = load_private_key_file(_support("test_ecdsa_384.key"))
+        self.assertIsInstance(key, ECDSAKey)
+
+    def test_autodetect_rsa(self):
+        key = load_private_key_file(_support("test_rsa.key"))
+        self.assertIsInstance(key, RSAKey)
+
+    def test_autodetect_dsa(self):
+        key = load_private_key_file(_support("test_dss.key"))
+        self.assertIsInstance(key, DSSKey)
+
+    def test_autodetect_password(self):
+        key = load_private_key_file(
+            _support("test_rsa_password.key"), password="television"
+        )
+        self.assertIsInstance(key, RSAKey)
+
+    def test_autodetect_string(self):
+        key = load_private_key(ECDSA_PRIVATE_OUT_256)
+        self.assertIsInstance(key, ECDSAKey)
+
+    def test_autodetect_unsupported(self):
+        with self.assertRaises(SSHException) as err:
+            load_private_key(UNSUPPORTED_PRIVATE_KEY)
+        self.assertEqual(
+            str(err.exception), "Unsupported key type UNSUPPORTED"
+        )
+
+    def test_wrong_key_type(self):
+        testcases = [
+            (
+                RSAKey,
+                _support("test_ed25519.key"),
+                "Expected key type ssh-rsa, got ssh-ed25519"
+            ),
+            (
+                Ed25519Key,
+                _support("test_rsa_password.key"),
+                "Expected key type OPENSSH, got RSA"
+            ),
+            (
+                ECDSAKey,
+                _support("test_dss.key"),
+                "Expected key type EC, got DSA"
+            ),
+            (
+                DSSKey,
+                _support("test_ecdsa_384_o.key"),
+                "Expected key type ssh-dss, got ecdsa-sha2-nistp384"
+            ),
+        ]
+        for cls, filename, errmsg in testcases:
+            with self.assertRaises(SSHException) as ctx:
+                cls(filename=filename, password=b"television")
+            self.assertEqual(str(ctx.exception), errmsg)
+
+
+class ParseTest(unittest.TestCase):
+    """
+    Testing mostly with old format and a password; other cases are a subset
+    of that.
+    """
+
+    @classmethod
+    def setUpClass(cls):
+        # Several tests rely on these fields.
+        cls.ecdsa_str = open(_support("test_ecdsa_password_256.key")).read().strip()
+        cls.ecdsa_parsed = PKey._parse_openssh_pkey(cls.ecdsa_str)
+
+    def test_parse_old_password(self):
+        """Old format with headers"""
+        pkformat, typ, headers, data = PKey._parse_openssh_pkey(self.ecdsa_str)
+
+        self.assertEqual(pkformat, PKey.FORMAT_ORIGINAL)
+        self.assertEqual(typ, ECDSAKey.LEGACY_TYPE)
+        self.assertEqual(headers, {
+            "dek-info": "AES-128-CBC,EEB56BC745EDB2DE04FC3FE1F8DA387E",
+            "proc-type": "4,ENCRYPTED"
+        })
+        self.assertIsInstance(data, bytes)
+        self.assertEqual(len(data), 128)
+
+    def test_parse_old_plain(self):
+        key_str = open(_support("test_rsa.key")).read()
+        pkformat, typ, headers, data = PKey._parse_openssh_pkey(key_str)
+
+        self.assertEqual(pkformat, PKey.FORMAT_ORIGINAL)
+        self.assertEqual(typ, RSAKey.LEGACY_TYPE)
+        self.assertEqual(headers, {})
+        self.assertIsInstance(data, bytes)
+        self.assertEqual(len(data), 606)
+
+    def test_parse_new(self):
+        """In new format, parsing is same for encryped and un-encrypted."""
+        key_str = open(_support("test_ed25519.key")).read()
+        pkformat, typ, headers, data = PKey._parse_openssh_pkey(key_str)
+
+        self.assertEqual(pkformat, PKey.FORMAT_OPENSSH)
+        self.assertEqual(typ, "OPENSSH")
+        self.assertEqual(headers, {})
+        self.assertIsInstance(data, bytes)
+        self.assertEqual(len(data), 266)
+
+    def test_parse_whitespace(self):
+        """In some places, extraneous whitespace should not affect parsing"""
+        key_str = self.ecdsa_str
+        key_str = key_str.replace("\n", "  \t\n")
+        key_str = key_str.replace(": ", ": \t ")
+        self.assertEqual(PKey._parse_openssh_pkey(key_str), self.ecdsa_parsed)
+
+    def test_parse_crlf(self):
+        """Test handling of Windows newlines"""
+        key_str = self.ecdsa_str
+        key_str = key_str.replace("\n", "\r\n")
+        self.assertEqual(PKey._parse_openssh_pkey(key_str), self.ecdsa_parsed)
+
+    def test_parse_junk(self):
+        """Non-key data is allowed before and after the key itself"""
+        key_str = '\n'.join([
+            "BLA bla BLA",
+            "Lots of junk",
+            self.ecdsa_str.strip(),
+            "It's your lucky day, we even have trailing junk!"
+        ])
+        self.assertEqual(PKey._parse_openssh_pkey(key_str), self.ecdsa_parsed)
+
+    def test_parse_begin_line(self):
+        """BEGIN tag must be on a line by itself"""
+        key_str = "BLA" + self.ecdsa_str
+        with self.assertRaises(SSHException):
+            PKey._parse_openssh_pkey(key_str)
+
+    def test_parse_end_line(self):
+        """END tag must be on a line by itself"""
+        key_str = self.ecdsa_str.strip() + "BLA"
+        with self.assertRaises(SSHException):
+            PKey._parse_openssh_pkey(key_str)
+
+    def test_bad_base64(self):
+        with self.assertRaises(SSHException) as ctx:
+            PKey._parse_openssh_pkey(INVALID_BASE64_KEY)
+        self.assertEqual(
+            str(ctx.exception), "base64 decoding error: Incorrect padding"
+        )
+
+    def test_parse_empty(self):
+        with self.assertRaises(SSHException) as ctx:
+            PKey._parse_openssh_pkey("")
+        self.assertEqual(str(ctx.exception), "not a valid private key file")
