@@ -27,7 +27,10 @@ import time
 import threading
 import random
 import unittest
-from mock import Mock
+try:
+    from unittest.mock import Mock
+except ImportError:
+    from mock import Mock
 
 from paramiko import (
     Transport, SecurityOptions, ServerInterface, RSAKey, SSHException,
@@ -717,7 +720,7 @@ class TransportTest(unittest.TestCase):
         class SendThread(threading.Thread):
             def __init__(self, chan, iterations, done_event):
                 threading.Thread.__init__(self, None, None, self.__class__.__name__)
-                self.daemon = True
+                self.setDaemon(True)
                 self.chan = chan
                 self.iterations = iterations
                 self.done_event = done_event
@@ -738,7 +741,7 @@ class TransportTest(unittest.TestCase):
         class ReceiveThread(threading.Thread):
             def __init__(self, chan, done_event):
                 threading.Thread.__init__(self, None, None, self.__class__.__name__)
-                self.daemon = True
+                self.setDaemon(True)
                 self.chan = chan
                 self.done_event = done_event
                 self.watchdog_event = threading.Event()
